# Contributing to Nodeology

Thank you for your interest in contributing to Nodeology! This document provides guidelines and information about contributing to the project.

## Ways to Contribute

### Code Contributions

1. **Core Framework**

   - Bug fixes and improvements
   - Performance optimizations
   - New features
   - Test coverage

2. **Pre-built Components**
   - New node types
   - State definitions
   - Workflow templates
   - Domain-specific tools

### Documentation

- API documentation
- Usage examples
- Tutorials
- Best practices

### Research Collaborations

1. **Workflow Patterns**

   - Novel automation patterns
   - Optimization strategies
   - Human-AI interaction interfaces
   - Error handling approaches

2. **Scientific Integration**

   - Domain-specific applications
   - Instrument interfaces
   - Data processing pipelines
   - Analysis tools

3. **Evaluation Methods**

   - Benchmark development
   - Performance metrics
   - Reliability assessment
   - Comparison frameworks

4. **AI Integration**
   - Prompt optimization
   - Model evaluation
   - Hybrid processing
   - Knowledge integration

## Getting Started

1. **Development Setup**

   ```bash
   # Clone repository
   git clone https://github.com/xyin-anl/nodeology.git
   cd nodeology

   # Create virtual environment using venv or conda
   python -m venv venv
   source venv/bin/activate

   # Install dependencies
   pip install -r requirements.txt

   # Install pytest
   pip install pytest

   # Run tests
   pytest tests/
   ```

## Guidelines

1. **Code Contributions**

   - Fork repository
   - Create feature branch
   - Submit pull request
   - Use `black` formatter
   - Write clear commit messages
   - Add unit tests
   - Update/create documentation if possible
   - Include example usage if possible

2. **Documentation**

   - Clear, concise writing
   - Practical examples
   - Proper formatting
   - Complete coverage

3. **Community**
   - Be respectful
   - Provide constructive feedback
   - Help new users
   - Share knowledge
